@@ -29,6 +29,7 @@
 CL_API_ENTRY cl_int
 POname (clCommandSVMMemcpyKHR) (
     cl_command_buffer_khr command_buffer, cl_command_queue command_queue,
+    const cl_command_properties_khr* properties,
     void *dst_ptr, const void *src_ptr, size_t size,
     cl_uint num_sync_points_in_wait_list,
     const cl_sync_point_khr *sync_point_wait_list,
@@ -36,12 +37,12 @@ POname (clCommandSVMMemcpyKHR) (
     cl_mutable_command_khr *mutable_handle) CL_API_SUFFIX__VERSION_2_0
 {
   cl_int errcode;
-
   CMDBUF_VALIDATE_COMMON_HANDLES;
+  SETUP_MUTABLE_HANDLE;
 
-  return pocl_svm_memcpy_common (command_buffer, command_queue,
-                                 CL_COMMAND_SVM_MEMCPY, dst_ptr, src_ptr, size,
-                                 num_sync_points_in_wait_list, NULL, NULL,
-                                 sync_point_wait_list, sync_point, NULL);
+  return pocl_svm_memcpy_common (
+    command_buffer, command_queue, CL_COMMAND_SVM_MEMCPY, dst_ptr, src_ptr,
+    size, num_sync_points_in_wait_list, NULL, NULL, sync_point_wait_list,
+    sync_point, mutable_handle);
 }
 POsym (clCommandSVMMemcpyKHR)
