@@ -57,7 +57,8 @@ CL_API_SUFFIX__VERSION_1_0
   desc.num_samples = 0;
 
   return pocl_create_image_internal (context, flags, &format, &desc, NULL,
-                                     errcode_ret, target, miplevel, texture);
+                                     errcode_ret, target, miplevel, texture,
+                                     NULL, NULL);
 
 #else
 
