@@ -21,27 +21,17 @@
    IN THE SOFTWARE.
 */
 
-#include <CL/cl_ext.h>
 
+#include "pocl.h"
 #include "pocl_cl.h"
 #include "pocl_mem_management.h"
 #include "pocl_shared.h"
 #include "pocl_util.h"
 
-static void CL_CALLBACK
-buffer_finished_callback (cl_event event, cl_int event_command_status,
-                          void *user_data)
-{
-  cl_command_buffer_khr command_buffer = (cl_command_buffer_khr)user_data;
-  POCL_LOCK (command_buffer->mutex);
-  command_buffer->pending -= 1;
-  if (command_buffer->pending == 0)
-    command_buffer->state = CL_COMMAND_BUFFER_STATE_EXECUTABLE_KHR;
-  POCL_UNLOCK (command_buffer->mutex);
-  POname (clReleaseCommandBufferKHR) (command_buffer);
-}
+#include <CL/cl_ext.h>
 
-CL_API_ENTRY cl_int
+CL_API_ENTRY
+cl_int CL_API_CALL
 POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
                                     cl_command_queue *queues,
                                     cl_command_buffer_khr command_buffer,
@@ -129,23 +119,55 @@ POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
   POCL_UNLOCK (command_buffer->mutex);
   POCL_RETURN_ERROR_COND ((!is_ready), CL_INVALID_OPERATION);
 
-  /* Submit to queue(s) */
-  if (num_used_queues == 1 && used_queues[0]->device->ops->run_command_buffer)
+  int runs_natively
+    = (!command_buffer->is_multi_device)
+      && used_queues[0]->device->native_command_buffers;
+  /* Submit to queue(s) directly if device supports cmdbufs natively */
+  if (runs_natively)
     {
       /* TODO: add base event id & increment global event id counter by number
        * of commands generated by the buffer */
-      return used_queues[0]->device->ops->run_command_buffer (
-        used_queues[0]->device, command_buffer);
+      /* TODO: figure out a way to handle migrations effectively. We can't
+       * change the cmd buffer once it's finished recording, as changing would
+       * also require changing the hardware resource (e.g. L0 command list),
+       * which would negate the point of using cmdbufs in the 1st place. */
+      _cl_command_node *node = NULL;
+      cl_event final_ev;
+      errcode = pocl_create_command (
+        &node, used_queues[0], CL_COMMAND_COMMAND_BUFFER_KHR, &final_ev,
+        num_events_in_wait_list, event_wait_list,
+        pocl_deep_copy_migration_info_list (command_buffer->migr_infos, 0));
+      if (errcode != CL_SUCCESS)
+        {
+          pocl_mem_manager_free_command (node);
+          return errcode;
+        }
+
+      final_ev->reset_command_buffer = CL_TRUE;
+      final_ev->command_buffer = command_buffer;
+      node->command.replay.buffer = command_buffer;
+
+      if (event_p != NULL)
+        *event_p = final_ev;
+      else
+        POname (clReleaseEvent) (final_ev);
+
+      POname (clRetainCommandBufferKHR) (command_buffer);
+      pocl_command_enqueue (used_queues[0], node);
+      return errcode;
     }
+
   /* Submit individual commands manually */
   else
     {
-      _cl_command_node *cmd;
-
-      cl_event syncpoints[command_buffer->num_syncpoints];
+      _cl_command_node *cmd = NULL;
+      cl_uint num_syncpoints = command_buffer->num_syncpoints;
+      cl_event *syncpoints = alloca (num_syncpoints * sizeof (cl_event));
+      memset (syncpoints, 0, num_syncpoints * sizeof (cl_event));
       cl_event *deps = (cl_event *)alloca (
-        sizeof (cl_event)
-        * (command_buffer->num_syncpoints + num_events_in_wait_list));
+        sizeof (cl_event) * (num_syncpoints + num_events_in_wait_list));
+      memset (deps, 0,
+              sizeof (cl_event) * (num_syncpoints + num_events_in_wait_list));
 
       unsigned sync_id = 0;
       LL_FOREACH (command_buffer->cmds, cmd)
@@ -166,25 +188,19 @@ POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
           }
 
         _cl_command_node *node = NULL;
-        char *readonly_flag_list = NULL;
-        cl_mem *memobj_list = NULL;
-        if (cmd->memobj_count != 0)
-          {
-            readonly_flag_list = malloc (cmd->memobj_count);
-            memcpy (readonly_flag_list, cmd->readonly_flag_list,
-                    cmd->memobj_count);
-            memobj_list = malloc (sizeof (cl_mem) * cmd->memobj_count);
-            memcpy (memobj_list, cmd->memobj_list,
-                    sizeof (cl_mem) * cmd->memobj_count);
-          }
+        /* The migration infos in the recorded command should not be touched.
+           When executing we clone them in order for the implicit migration
+           code to be able to insert new implicit sub-buffers and to make the
+           list freeable (and its buffers releasable) after the copied command
+           instance finishes.  The recorded command's migration infos are
+           released in clReleaseCommandBufferKHR's
+           pocl_mem_manager_free_command call. */
+
         errcode = pocl_create_command (
           &node, used_queues[cmd->queue_idx], cmd->type, &syncpoints[sync_id],
-          j, deps, cmd->memobj_count, memobj_list, readonly_flag_list);
+          j, deps, pocl_deep_copy_migration_info_list (cmd->migr_infos, 0));
         ++sync_id;
 
-        POCL_MEM_FREE (readonly_flag_list);
-        POCL_MEM_FREE (memobj_list);
-
         if (errcode != CL_SUCCESS)
           {
             POCL_MSG_ERR ("Failed to instantiate recorded command: %i\n",
@@ -193,7 +209,7 @@ POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
             return errcode;
           }
 
-        errcode = pocl_copy_event_node (node, cmd);
+        errcode = pocl_copy_command_node (node, cmd);
 
         if (errcode != CL_SUCCESS)
           {
@@ -213,8 +229,8 @@ POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
       _cl_command_node *node = NULL;
       cl_event final_ev;
       errcode = pocl_create_command (
-        &node, used_queues[0], CL_COMMAND_COMMAND_BUFFER_KHR, &final_ev,
-        command_buffer->num_syncpoints, syncpoints, 0, NULL, NULL);
+        &node, used_queues[0], CL_COMMAND_MARKER, &final_ev,
+        command_buffer->num_syncpoints, syncpoints, NULL);
       if (errcode != CL_SUCCESS)
         {
           pocl_mem_manager_free_command (node);
@@ -232,16 +248,8 @@ POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
                  : 0;
         }
 
-      errcode = POname (clSetEventCallback) (final_ev, CL_COMPLETE,
-                                             buffer_finished_callback,
-                                             (void *)command_buffer);
-      if (errcode != CL_SUCCESS)
-        {
-          POCL_MSG_ERR ("Failed to set command buffer cleanup callback\n");
-          POname (clReleaseEvent) (final_ev);
-          pocl_mem_manager_free_command (node);
-          return errcode;
-        }
+      final_ev->reset_command_buffer = CL_TRUE;
+      final_ev->command_buffer = command_buffer;
 
       if (event_p != NULL)
         *event_p = final_ev;
@@ -254,8 +262,8 @@ POname (clEnqueueCommandBufferKHR) (cl_uint num_queues,
         }
       POname (clRetainCommandBufferKHR) (command_buffer);
       pocl_command_enqueue (used_queues[0], node);
-
-      return CL_SUCCESS;
     }
+
+  return CL_SUCCESS;
 }
 POsym (clEnqueueCommandBufferKHR)
