@@ -24,8 +24,7 @@
 
 #include "pocl_debug.h"
 #include "pocl_util.h"
-
-extern unsigned long usm_buffer_c;
+#include "utlist.h"
 
 /* get & retain all last-events of all command queues of the context */
 static int
@@ -55,7 +54,6 @@ pocl_get_last_events (cl_context context, cl_event **last_events,
 
   for (unsigned i = 0; i < context->num_devices; ++i)
     {
-      cl_device_id dev = context->devices[i];
       if (context->default_queues && context->default_queues[i])
         {
           POCL_LOCK_OBJ (context->default_queues[i]);
@@ -105,15 +103,13 @@ pocl_mem_free_intel (cl_context context, void *usm_pointer, cl_bool blocking)
 
   if (blocking == CL_FALSE)
     {
-      context->usm_allocdev->ops->usm_free (context->usm_allocdev,
-                                            usm_pointer);
+      item->device->ops->usm_free (item->device, usm_pointer);
     }
   else
     {
       /* if the device implements blocking free callback */
-      if (context->usm_allocdev->ops->usm_free_blocking)
-        context->usm_allocdev->ops->usm_free_blocking (context->usm_allocdev,
-                                                       usm_pointer);
+      if (item->device->ops->usm_free_blocking)
+        item->device->ops->usm_free_blocking (item->device, usm_pointer);
       else
         {
           /* otherwise wait for all queues in the context */
@@ -124,7 +120,10 @@ pocl_mem_free_intel (cl_context context, void *usm_pointer, cl_bool blocking)
                                           &last_event_count);
           POCL_UNLOCK_OBJ (context);
           if (err != CL_SUCCESS)
-            return err;
+            {
+              assert (last_events == NULL);
+              return err;
+            }
           if (last_event_count > 0)
             {
               POname (clWaitForEvents) (last_event_count, last_events);
@@ -133,8 +132,8 @@ pocl_mem_free_intel (cl_context context, void *usm_pointer, cl_bool blocking)
                   POname (clReleaseEvent) (last_events[i]);
                 }
             }
-          context->usm_allocdev->ops->usm_free (context->usm_allocdev,
-                                                usm_pointer);
+          free (last_events);
+          item->device->ops->usm_free (item->device, usm_pointer);
         }
     }
 
