@@ -23,8 +23,7 @@
 
 #include "pocl_util.h"
 #include "pocl_debug.h"
-
-extern unsigned long svm_buffer_c;
+#include "utlist.h"
 
 CL_API_ENTRY void CL_API_CALL
 POname(clSVMFree)(cl_context context,
