@@ -230,10 +230,6 @@ void pocl_openasip_write_kernel_descriptor(char *content, size_t content_size,
            "void _pocl_kernel_%s"
            "_workgroup(uint8_t* args, uint8_t*, "
            "uint32_t, uint32_t, uint32_t);\n"
-           "void _pocl_kernel_%s"
-           "_workgroup_fast(uint8_t* args, uint8_t*, "
-           "uint32_t, uint32_t, uint32_t);\n"
-
            "void %s"
            "_workgroup_argbuffer("
            "uint8_t "
@@ -522,6 +518,7 @@ void pocl_almaif_openasip_produce_standalone_program(AlmaifData *D,
   out << "#include \"almaif-tce-device-defs.h\"" << std::endl << std::endl;
 
   out << "#undef ALIGN4" << std::endl;
+  // Using __attribute__ is fine here as the code is passed to clang.
   out << "#define ALIGN4 __attribute__ ((aligned (4)))" << std::endl;
 
   /* The standalone binary shall have the same input data as in the original
