@@ -185,6 +185,12 @@ pocl_tce_device_hash (const char *adf_file, const char *llvm_triplet,
   return 0;
 }
 
+static const char *ttasim_llvm_intrin_replace(const char *IName, size_t Size) {
+  std::string Intrin(IName, Size);
+  if (Intrin == "llvm.memcpy.p1.p2.i32")
+    return "__pocl_tce_memcpy_p1_p2_i32";
+  return nullptr;
+}
 
 class TTASimDevice : public TCEDevice {
 public:
@@ -224,6 +230,7 @@ public:
        */
       dev->llvm_fp_contract_mode = "off";
     }
+    dev->llvm_intrin_replace = ttasim_llvm_intrin_replace;
     dev->kernellib_fallback_name = NULL;
     dev->kernellib_subdir = "tce";
 
@@ -256,9 +263,9 @@ public:
     POCL_UNLOCK(lock);
     POCL_JOIN_THREAD(ttasim_thread);
 
-    POCL_FAST_LOCK(wq_lock);
+    POCL_LOCK(wq_lock);
     POCL_SIGNAL_COND(wakeup_cond);
-    POCL_FAST_UNLOCK(wq_lock);
+    POCL_UNLOCK(wq_lock);
     POCL_JOIN_THREAD(driver_thread);
   }
 
@@ -340,6 +347,7 @@ public:
     out << "#include <pocl_device.h>" << std::endl << std::endl;
 
     out << "#undef ALIGN4" << std::endl;
+    // The __attribute__s are fine here as the code is passed to clang.
     out << "#define ALIGN4 __attribute__ ((aligned (4)))" << std::endl;
     out << "#define __local__ __attribute__((address_space(" <<  TTA_ASID_LOCAL<< ")))" << std::endl;
     out << "#define __global__ __attribute__((address_space(" << TTA_ASID_GLOBAL << ")))" << std::endl;
@@ -375,7 +383,7 @@ public:
             cl_mem m = *(cl_mem *)(al->value);
             void *p = m->device_ptrs[parent->global_mem_id].mem_ptr;
             chunk_info_t *ci = (chunk_info_t *)p;
-            unsigned start_addr = ci->start_address + al->offset;
+            unsigned start_addr = ci->start_address;
             unsigned size = ci->size;
 
             out << "__global__ ALIGN4 char buffer_" << std::hex << start_addr
@@ -633,7 +641,6 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
   dev->type = CL_DEVICE_TYPE_GPU;
   dev->max_compute_units = 1;
   dev->max_work_item_dimensions = 3;
-  dev->device_side_printf = 0;
 
   int max_wg
       = pocl_get_int_option ("POCL_MAX_WORK_GROUP_SIZE", DEFAULT_WG_SIZE);
@@ -669,7 +676,7 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
   dev->max_clock_frequency = 100;
   dev->image_support = CL_FALSE;
   dev->single_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
-  dev->double_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
+  dev->double_fp_config = 0;
   dev->global_mem_cache_type = CL_NONE;
   dev->local_mem_type = CL_GLOBAL;
   dev->error_correction_support = CL_FALSE;
@@ -694,7 +701,7 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
   dev->spmd = CL_FALSE;
   dev->run_workgroup_pass = CL_TRUE;
   dev->execution_capabilities = CL_EXEC_KERNEL;
-  dev->queue_properties = CL_QUEUE_PROFILING_ENABLE;
+  dev->on_host_queue_props = CL_QUEUE_PROFILING_ENABLE;
   dev->vendor = "TTA-Based Co-design Environment";
   dev->profile = "EMBEDDED_PROFILE";
   dev->extensions = TCE_DEVICE_EXTENSIONS;
@@ -733,17 +740,18 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
   dev->mem_base_addr_align = 128;
   dev->min_data_type_align_size = 128;
 
-  dev->device_side_printf = 1;
+  dev->device_side_printf = CL_TRUE;
   dev->printf_buffer_size = PRINTF_BUFFER_SIZE;
   TTASimDevice *d = (TTASimDevice *)dev->data;
   d->printf_buffer =
       pocl_alloc_buffer_from_region(&d->global_mem, dev->printf_buffer_size);
-  assert(d->printf_buffer);
   d->printf_position_chunk = pocl_alloc_buffer_from_region(&d->global_mem, 4);
-  if (d->printf_position_chunk == NULL) {
-    POCL_ABORT("TTASIM: Can't allocate 4 bytes for printf index\n");
+  if (d->printf_position_chunk == NULL || d->printf_buffer == NULL) {
+    POCL_MSG_ERR("TTASIM: Can't allocate 4 bytes for printf index, disabled "
+                 "device-side printf\n");
+    dev->device_side_printf = CL_FALSE;
   }
-  
+
   dev->max_parameter_size = 1024;
 
   return CL_SUCCESS;
