@@ -1,7 +1,7 @@
 // Implementation of LLVMUtils, useful common LLVM-related functionality.
 //
 // Copyright (c) 2013-2019 Pekka Jääskeläinen
-//               2023 Pekka Jääskeläinen / Intel Finland Oy
+//               2023-2024 Pekka Jääskeläinen / Intel Finland Oy
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,6 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include "AllocasToEntry.h"
 #include "AutomaticLocals.h"
 #include "BarrierTailReplication.h"
-#include "BreakConstantGEPs.h"
 #include "CanonicalizeBarriers.h"
 #include "DebugHelpers.h"
 #include "Flatten.hh"
@@ -48,20 +47,21 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include "ImplicitLoopBarriers.h"
 #include "InlineKernels.hh"
 #include "IsolateRegions.h"
+#include "KernelCompilerUtils.h"
 #include "LoopBarriers.h"
 #include "MinLegalVecSize.hh"
-#include "OptimizeWorkItemFuncCalls.h"
 #include "OptimizeWorkItemGVars.h"
 #include "PHIsToAllocas.h"
 #include "ParallelRegion.h"
 #include "RemoveBarrierCalls.h"
+#include "SanitizeUBofDivRem.h"
 #include "SubCFGFormation.h"
+#include "UnreachablesToReturns.h"
 #include "VariableUniformityAnalysis.h"
 #include "WorkItemAliasAnalysis.h"
 #include "Workgroup.h"
 #include "WorkitemHandlerChooser.h"
 #include "WorkitemLoops.h"
-#include "WorkitemReplication.h"
 
 #include "LLVMUtils.h"
 POP_COMPILER_DIAGS
@@ -186,7 +186,11 @@ void breakConstantExpressions(llvm::Value *Val, llvm::Function *Func) {
 
       // Convert this constant expression to an instruction.
       llvm::Instruction *I = CE->getAsInstruction();
+#if LLVM_MAJOR < 20
       I->insertBefore(&*Func->begin()->begin());
+#else
+      I->insertBefore(Func->begin()->begin());
+#endif
       CE->replaceAllUsesWith(I);
       CE->destroyConstant();
     }
@@ -250,8 +254,7 @@ isAutomaticLocal(llvm::Function *F, llvm::GlobalVariable &Var) {
   std::string FuncName = F->getName().str();
   if (!llvm::isa<llvm::PointerType>(Var.getType()) || Var.isConstant())
     return false;
-  if (Var.getName().startswith(FuncName + ".")) {
-    assert(isGVarUsedByFunction(&Var, F) == true);
+  if (Var.getName().starts_with(FuncName + ".")) {
     return true;
   }
 
@@ -407,6 +410,20 @@ void setFuncArgAddressSpaceMD(llvm::Function *F, unsigned ArgIndex,
   F->setMetadata(MDKind, MDNode::get(F->getContext(), AddressQuals));
 }
 
+void markFunctionAlwaysInline(llvm::Function *F) {
+  F->removeFnAttr(Attribute::NoInline);
+  F->removeFnAttr(Attribute::OptimizeNone);
+  F->addFnAttr(Attribute::AlwaysInline);
+  // remove noInline from the callsite. otherwise it could cause alwaysInline
+  // pass to skip the inlining
+  for (auto U: F->users()) {
+    if (CallInst *CI = dyn_cast<CallInst>(U)) {
+      CI->removeFnAttr(Attribute::NoInline);
+      CI->removeFnAttr(Attribute::OptimizeNone);
+    }
+  }
+}
+
 // Returns true in case the given function is a kernel that
 // should be processed by the kernel compiler.
 bool isKernelToProcess(const llvm::Function &F) {
@@ -421,7 +438,7 @@ bool isKernelToProcess(const llvm::Function &F) {
     return false;
   if (!F.hasName())
     return false;
-  if (F.getName().startswith("@llvm"))
+  if (F.getName().starts_with("@llvm"))
     return false;
 
   NamedMDNode *kernels = m->getNamedMetadata("opencl.kernels");
@@ -449,68 +466,6 @@ bool isKernelToProcess(const llvm::Function &F) {
   return false;
 }
 
-//#define DEBUG_UNREACHABLE_SWITCH_REMOVAL
-
-void removeUnreachableSwitchCases(llvm::Function &F) {
-  std::vector<BasicBlock *> BBsToDel;
-  for (Function::iterator FI = F.begin(), FE = F.end(); FI != FE; ++FI) {
-    BasicBlock *BB = &*FI;
-
-    if (BB->hasName() && BB->getName().startswith("default.unreachable")) {
-#ifdef DEBUG_UNREACHABLE_SWITCH_REMOVAL
-      std::cerr << "##################################################\n";
-      std::cerr << "### converting unreachable block: " << (void *)BB << "\n";
-#endif
-      BBsToDel.push_back(BB);
-
-      std::set<SwitchInst *> SwUsers;
-      for (auto U : BB->users()) {
-        if (SwitchInst *SwI = dyn_cast<SwitchInst>(U)) {
-          SwUsers.insert(SwI);
-        } else {
-#ifdef DEBUG_UNREACHABLE_SWITCH_REMOVAL
-          // we can ignore BBlocks with a single "br label default.unreachable"
-          if (!isa<BranchInst>(U)) {
-            std::cerr << "Unhandled unreachable user:\n";
-            U->dump();
-          }
-#endif
-        }
-      }
-
-      for (SwitchInst *SwI : SwUsers) {
-#ifdef DEBUG_UNREACHABLE_SWITCH_REMOVAL
-        std::cerr << "Found a switch user, replacing the unr label:\n";
-        SwI->dump();
-#endif
-        if (SwI->getDefaultDest() == BB) {
-#ifdef DEBUG_UNREACHABLE_SWITCH_REMOVAL
-          std::cerr << "... default switch user is the unr BB\n";
-#endif
-          // remove the last case, and make its BB as the default
-          auto FinalCaseIt = std::prev(SwI->case_end());
-          BasicBlock *FinalBB = FinalCaseIt->getCaseSuccessor();
-          SwI->removeCase(FinalCaseIt);
-          SwI->setDefaultDest(FinalBB);
-#ifdef DEBUG_UNREACHABLE_SWITCH_REMOVAL
-          std::cerr << "Final fixed switch:\n";
-          SwI->dump();
-#endif
-        } else {
-#ifdef DEBUG_UNREACHABLE_SWITCH_REMOVAL
-          std::cerr << "Unhandled switch, the default branch is not unr:\n";
-          SwI->dump();
-#endif
-        }
-      }
-    }
-  }
-
-  for (auto BB : BBsToDel) {
-    BB->eraseFromParent();
-  }
-}
-
 // Returns true in case the given function is a kernel with work-group
 // barriers inside it.
 bool hasWorkgroupBarriers(const llvm::Function &F) {
@@ -532,6 +487,90 @@ bool hasWorkgroupBarriers(const llvm::Function &F) {
   return false;
 }
 
+// walks through a Module's global variables,
+// determines which ones are OpenCL program-scope variables
+// and checks all of those have definitions
+bool areAllGvarsDefined(llvm::Module *Program, std::string &log,
+                        std::set<llvm::GlobalVariable *> &GVarSet,
+                        unsigned DeviceLocalAS) {
+
+  bool FoundAllReferences = true;
+
+  for (GlobalVariable &GVar : Program->globals()) {
+
+    if (isProgramScopeVariable(GVar, DeviceLocalAS)) {
+
+      assert(GVar.hasName());
+      // adding GV declarations to the module also changes
+      // the global iteration to include them
+      if (GVarSet.count(&GVar) != 0)
+        continue;
+
+      if (GVar.isDeclaration()) {
+        log.append("Undefined reference for program scope variable: ");
+        log.append(GVar.getName().data());
+        log.append("\n");
+        FoundAllReferences = false;
+      } else {
+        GVarSet.insert(&GVar);
+        // std::cerr << "**************************\n";
+        // GVar.dump();
+        // std::cerr << "**************************\n";
+      }
+    }
+  }
+
+  return FoundAllReferences;
+}
+
+// for a set of program scope variables,
+// calculate their offsets & sizes for later replacement with
+// indexing into a single large buffer
+// @returns the total size of all variables
+size_t
+calculateGVarOffsetsSizes(const DataLayout &DL,
+                          std::map<GlobalVariable *, uint64_t> &GVarOffsets,
+                          std::set<llvm::GlobalVariable *> &GVarSet) {
+
+  std::map<GlobalVariable *, uint64_t> GVarSizes;
+
+  // offset into the storage buffer for all of this program's global variables
+  size_t CurrentOffset = 0;
+
+  for (GlobalVariable *GVar : GVarSet) {
+    assert(GVar->hasInitializer());
+
+    // if the current offset into the buffer is not aligned enough, fix it
+#if LLVM_MAJOR < 15
+    uint64_t GVarAlign = GVar->getAlignment();
+#else
+    Align GVarA = GVar->getAlign().valueOrOne();
+    uint64_t GVarAlign = GVarA.value();
+#endif
+
+    if (GVarAlign > 0 && CurrentOffset % GVarAlign) {
+      CurrentOffset |= (GVarAlign - 1);
+      ++CurrentOffset;
+    }
+    GVarOffsets[GVar] = CurrentOffset;
+
+    // add to the offset the required amount of storage for the global variable
+    TypeSize GVSize = DL.getTypeAllocSize(GVar->getValueType());
+    assert(GVSize.isScalable() == false);
+    GVarSizes[GVar] = GVSize.getFixedValue();
+    CurrentOffset += GVarSizes[GVar];
+
+#ifdef POCL_DEBUG_PROGVARS
+    std::cerr << "@@@ GlobalVar: " << GVar->getName().str()
+              << "\n   OFFSET: " << GVarOffsets[GVar]
+              << "\n   SIZE: " << GVarSizes[GVar] << "\n";
+#endif
+  }
+
+  size_t TotalSize = CurrentOffset;
+  return TotalSize;
+}
+
 const char *WorkgroupVariablesArray[NumWorkgroupVariables+1] = {"_local_id_x",
                                     "_local_id_y",
                                     "_local_id_z",
@@ -548,6 +587,9 @@ const char *WorkgroupVariablesArray[NumWorkgroupVariables+1] = {"_local_id_x",
                                     "_global_offset_x",
                                     "_global_offset_y",
                                     "_global_offset_z",
+                                    "_global_id_x",
+                                    "_global_id_y",
+                                    "_global_id_z",
                                     "_pocl_sub_group_size",
                                     PoclGVarBufferName,
                                     NULL};
@@ -556,30 +598,44 @@ const std::vector<std::string>
     WorkgroupVariablesVector(WorkgroupVariablesArray,
                              WorkgroupVariablesArray+NumWorkgroupVariables);
 
-const char *WIFuncNameArray[NumWIFuncNames] = {"_Z13get_global_idj",
-                                               "_Z17get_global_offsetj",
-                                               "_Z15get_global_sizej",
-                                               "_Z12get_group_idj",
-                                               "_Z12get_local_idj",
-                                               "_Z14get_local_sizej",
-                                               "_Z23get_enqueued_local_sizej",
-                                               "_Z14get_num_groupsj",
-                                               "_Z20get_global_linear_idv",
-                                               "_Z19get_local_linear_idv",
-                                               "_Z12get_work_dimv"};
+const char *WIFuncNameArray[] = {
+    GID_BUILTIN_NAME,        GOFF_BUILTIN_NAME,    GS_BUILTIN_NAME,
+    GROUP_ID_BUILTIN_NAME,   LID_BUILTIN_NAME,     LS_BUILTIN_NAME,
+    ENQUEUE_LS_BUILTIN_NAME, NGROUPS_BUILTIN_NAME, GLID_BUILTIN_NAME,
+    LLID_BUILTIN_NAME,       WDIM_BUILTIN_NAME,    "__pocl_work_group_alloca"};
+
+constexpr unsigned NumWIFuncNames =
+    sizeof(WIFuncNameArray) / sizeof(const char *);
 
 const std::vector<std::string> WIFuncNameVec(WIFuncNameArray,
                                              WIFuncNameArray + NumWIFuncNames);
 
-#if LLVM_MAJOR >= MIN_LLVM_NEW_PASSMANAGER
+const char *DIFuncNameArray[NumDIFuncNames] = {GID_BUILTIN_NAME,
+                                               GOFF_BUILTIN_NAME,
+                                               GS_BUILTIN_NAME,
+                                               GROUP_ID_BUILTIN_NAME,
+                                               LID_BUILTIN_NAME,
+                                               LS_BUILTIN_NAME,
+                                               ENQUEUE_LS_BUILTIN_NAME,
+                                               NGROUPS_BUILTIN_NAME,
+                                               GLID_BUILTIN_NAME,
+                                               LLID_BUILTIN_NAME,
+                                               WDIM_BUILTIN_NAME,
+                                               "pocl_printf_alloc",
+                                               "pocl_printf_alloc_stub"};
+
+const std::vector<std::string> DIFuncNameVec(DIFuncNameArray,
+                                             DIFuncNameArray + NumDIFuncNames);
+
 // register all PoCL analyses & passes with an LLVM PassBuilder instance,
 // so that it can parse them from string representation
 void registerPassBuilderPasses(llvm::PassBuilder &PB) {
   AllocasToEntry::registerWithPB(PB);
   AutomaticLocals::registerWithPB(PB);
   BarrierTailReplication::registerWithPB(PB);
-  BreakConstantGEPs::registerWithPB(PB);
   CanonicalizeBarriers::registerWithPB(PB);
+  SanitizeUBofDivRem::registerWithPB(PB);
+  ConvertUnreachablesToReturns::registerWithPB(PB);
   FlattenAll::registerWithPB(PB);
   FlattenBarrierSubs::registerWithPB(PB);
   FlattenGlobals::registerWithPB(PB);
@@ -590,14 +646,12 @@ void registerPassBuilderPasses(llvm::PassBuilder &PB) {
   IsolateRegions::registerWithPB(PB);
   LoopBarriers::registerWithPB(PB);
   FixMinVecSize::registerWithPB(PB);
-  OptimizeWorkItemFuncCalls::registerWithPB(PB);
   OptimizeWorkItemGVars::registerWithPB(PB);
   PHIsToAllocas::registerWithPB(PB);
   RemoveBarrierCalls::registerWithPB(PB);
   SubCFGFormation::registerWithPB(PB);
   Workgroup::registerWithPB(PB);
   WorkitemLoops::registerWithPB(PB);
-  WorkitemReplication::registerWithPB(PB);
   PoCLCFGPrinter::registerWithPB(PB);
 }
 
@@ -606,6 +660,46 @@ void registerFunctionAnalyses(llvm::PassBuilder &PB) {
   WorkitemHandlerChooser::registerWithPB(PB);
   WorkItemAliasAnalysis::registerWithPB(PB);
 }
-#endif
+
+/**
+ * Returns the size_t for the current target.
+ */
+llvm::Type *SizeT(llvm::Module *M) {
+  unsigned long AddressBits;
+  getModuleIntMetadata(*M, "device_address_bits", AddressBits);
+  return IntegerType::get(M->getContext(), AddressBits);
+}
+
+bool isWorkitemFunctionWithOnlyCompilerExpandableCalls(
+    const llvm::Function &F) {
+
+  if (F.getName() != GID_BUILTIN_NAME && F.getName() != GS_BUILTIN_NAME &&
+      F.getName() != GROUP_ID_BUILTIN_NAME && F.getName() != LID_BUILTIN_NAME &&
+      F.getName() != LS_BUILTIN_NAME && F.getName() != NGROUPS_BUILTIN_NAME)
+    return false;
+
+  for (const auto &U : F.uses()) {
+    llvm::CallInst *Call = dyn_cast<llvm::CallInst>(U.getUser());
+    if (Call == nullptr)
+      continue;
+    if (!isCompilerExpandableWIFunctionCall(*Call))
+      return false;
+  }
+  return true;
+}
+
+bool isCompilerExpandableWIFunctionCall(const llvm::CallInst &Call) {
+  auto Callee = Call.getCalledFunction();
+  if (Callee == nullptr /* Inline asm? */)
+    return false;
+  if (Callee->getName() != GID_BUILTIN_NAME &&
+      Callee->getName() != GS_BUILTIN_NAME &&
+      Callee->getName() != GROUP_ID_BUILTIN_NAME &&
+      Callee->getName() != LID_BUILTIN_NAME &&
+      Callee->getName() != LS_BUILTIN_NAME &&
+      Callee->getName() != NGROUPS_BUILTIN_NAME)
+    return false;
+  return isa<llvm::ConstantInt>(Call.getArgOperand(0));
+}
 
 } // namespace pocl
