@@ -32,10 +32,13 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 #include "llvm/Transforms/Utils/Cloning.h"
 
-#include "ParallelRegion.h"
 #include "Barrier.h"
-#include "Kernel.h"
 #include "DebugHelpers.h"
+#include "Kernel.h"
+#include "KernelCompilerUtils.h"
+#include "LLVMUtils.h"
+#include "ParallelRegion.h"
+
 POP_COMPILER_DIAGS
 
 #include <algorithm>
@@ -58,10 +61,8 @@ using namespace pocl;
 
 int ParallelRegion::idGen = 0;
 
-ParallelRegion::ParallelRegion(int forcedRegionId) :
-  LocalIDXLoadInstr(NULL), LocalIDYLoadInstr(NULL), LocalIDZLoadInstr(NULL),
-  exitIndex_(0), entryIndex_(0), pRegionId(forcedRegionId)
-{
+ParallelRegion::ParallelRegion(int forcedRegionId)
+    : exitIndex_(0), entryIndex_(0), pRegionId(forcedRegionId) {
   if (forcedRegionId == -1)
     pRegionId = idGen++;
 }
@@ -148,7 +149,7 @@ ParallelRegion::replicate(ValueToValueMapTy &map,
 #ifdef DEBUG_REPLICATE
   Verify();
 #endif
-  LocalizeIDLoads();
+  localizeIDLoads();
 
   return new_region;
 }
@@ -295,22 +296,17 @@ ParallelRegion::insertLocalIdInit(llvm::BasicBlock* Entry,
 
   Module *M = Entry->getParent()->getParent();
 
-  unsigned long address_bits;
-  getModuleIntMetadata(*M, "device_address_bits", address_bits);
-
-  llvm::Type *SizeT = IntegerType::get(M->getContext(), address_bits);
-
-  GlobalVariable *GVX = M->getGlobalVariable(POCL_LOCAL_ID_X_GLOBAL);
+  GlobalVariable *GVX = M->getGlobalVariable(LID_G_NAME(0));
   if (GVX != NULL)
-      Builder.CreateStore(ConstantInt::get(SizeT, X), GVX);
+    Builder.CreateStore(ConstantInt::get(SizeT(M), X), GVX);
 
-  GlobalVariable *GVY = M->getGlobalVariable(POCL_LOCAL_ID_Y_GLOBAL);
+  GlobalVariable *GVY = M->getGlobalVariable(LID_G_NAME(1));
   if (GVY != NULL)
-      Builder.CreateStore(ConstantInt::get(SizeT, Y), GVY);
+    Builder.CreateStore(ConstantInt::get(SizeT(M), Y), GVY);
 
-  GlobalVariable *GVZ = M->getGlobalVariable(POCL_LOCAL_ID_Z_GLOBAL);
+  GlobalVariable *GVZ = M->getGlobalVariable(LID_G_NAME(2));
   if (GVZ != NULL)
-      Builder.CreateStore(ConstantInt::get(SizeT, Z), GVZ);
+    Builder.CreateStore(ConstantInt::get(SizeT(M), Z), GVZ);
 }
 
 void
@@ -346,37 +342,37 @@ ParallelRegion::dumpNames()
     std::cerr << std::endl;
 }
 
-ParallelRegion *
-ParallelRegion::Create(const SmallPtrSet<BasicBlock *, 8>& bbs, BasicBlock *entry, BasicBlock *exit)
-{
-  ParallelRegion *new_region = new ParallelRegion();
+ParallelRegion *ParallelRegion::Create(const SmallPtrSet<BasicBlock *, 8> &BBs,
+                                       BasicBlock *Entry, BasicBlock *Exit) {
+  ParallelRegion *NewRegion = new ParallelRegion();
 
-  assert (entry != NULL);
-  assert (exit != NULL);
+  assert(Entry != NULL);
+  assert(Exit != NULL);
 
   // This is done in two steps so order of the vector
   // is the same as original function order.
-  Function *F = entry->getParent();
+  Function *F = Entry->getParent();
   for (Function::iterator i = F->begin(), e = F->end(); i != e; ++i) {
-    BasicBlock *b = &*i;
-    for (SmallPtrSetIterator<BasicBlock *> j = bbs.begin(); j != bbs.end(); ++j) {
-      if (*j == b) {
-        new_region->push_back(&*i);
-        if (entry == *j)
-            new_region->setEntryBBIndex(new_region->size() - 1);
-        else if (exit == *j)
-            new_region->setExitBBIndex(new_region->size() - 1);
+    BasicBlock *B = &*i;
+    for (SmallPtrSetIterator<BasicBlock *> j = BBs.begin(); j != BBs.end();
+         ++j) {
+      if (*j == B) {
+        NewRegion->push_back(&*i);
+        if (Entry == *j)
+          NewRegion->setEntryBBIndex(NewRegion->size() - 1);
+        else if (Exit == *j)
+          NewRegion->setExitBBIndex(NewRegion->size() - 1);
         break;
       }
     }
   }
 
-  new_region->LocalizeIDLoads();
+  NewRegion->localizeIDLoads();
 #ifdef DEBUG_CREATE
-  assert(new_region->Verify());
+  assert(NewRegion->Verify());
 #endif
 
-  return new_region;
+  return NewRegion;
 }
 
 bool
@@ -387,7 +383,7 @@ ParallelRegion::Verify()
   // 2) Single outgoing edge from exit block
   //    (other outgoing edges allowed, will be purged in replicas).
   // 3) No barriers inside the region.
-  
+
   int entry_edges = 0;
 
   for (iterator i = begin(), e = end(); i != e; ++i) {
@@ -416,7 +412,7 @@ ParallelRegion::Verify()
         ++entry_edges;
       }
     }
-    
+
     // if (entry_edges != 1) {
     //   assert(0 && "Parallel regions must be single entry!");
     //   return false;
@@ -493,19 +489,18 @@ ParallelRegion::Verify()
  * vectorization easier. In this case using of IsLoadUnconditionallySafe
  * parameter will be skipped.
  */
-void
-ParallelRegion::AddParallelLoopMetadata(
+void ParallelRegion::addParallelLoopMetadata(
     llvm::MDNode *Identifier,
     std::function<bool(llvm::Instruction *)> IsLoadUnconditionallySafe) {
   for (iterator i = begin(), e = end(); i != e; ++i) {
-    BasicBlock* bb = *i;      
-    for (BasicBlock::iterator ii = bb->begin(), ee = bb->end();
-         ii != ee; ii++) {
+    BasicBlock *BB = *i;
+    for (BasicBlock::iterator ii = BB->begin(), ee = BB->end(); ii != ee;
+         ii++) {
       if (!ii->mayReadOrWriteMemory()) {
         continue;
       }
 
-      MDNode *NewMD = MDNode::get(bb->getContext(), Identifier);
+      MDNode *NewMD = MDNode::get(BB->getContext(), Identifier);
       MDNode *OldMD = ii->getMetadata(PARALLEL_MD_NAME);
       if (OldMD != nullptr) {
         NewMD = llvm::MDNode::concatenate(OldMD, NewMD);
@@ -515,51 +510,6 @@ ParallelRegion::AddParallelLoopMetadata(
   }
 }
 
-void
-ParallelRegion::AddIDMetadata(
-    llvm::LLVMContext& context, 
-    std::size_t x, 
-    std::size_t y, 
-    std::size_t z) {
-    int counter = 1;
-    Metadata *v1[] = {
-        MDString::get(context, "WI_region"),      
-        llvm::ConstantAsMetadata::get(
-          ConstantInt::get(Type::getInt32Ty(context), pRegionId))
-    };
-    MDNode* mdRegion = MDNode::get(context, v1);  
-    Metadata *v2[] = {
-        MDString::get(context, "WI_xyz"),      
-        llvm::ConstantAsMetadata::get(
-          ConstantInt::get(Type::getInt32Ty(context), x)),
-        llvm::ConstantAsMetadata::get(
-          ConstantInt::get(Type::getInt32Ty(context), y)),      
-        llvm::ConstantAsMetadata::get(
-          ConstantInt::get(Type::getInt32Ty(context), z))};
-    MDNode* mdXYZ = MDNode::get(context, v2);  
-    Metadata *v[] = {
-        MDString::get(context, "WI_data"),      
-        mdRegion,
-        mdXYZ};
-    MDNode* md = MDNode::get(context, v);              
-    
-    for (iterator i = begin(), e = end(); i != e; ++i) {
-      BasicBlock* bb = *i;
-      for (BasicBlock::iterator ii = bb->begin();
-            ii != bb->end(); ii++) {
-        Metadata *v3[] = {
-            MDString::get(context, "WI_counter"),      
-            llvm::ConstantAsMetadata::get(
-              ConstantInt::get(Type::getInt32Ty(context), counter))};
-        MDNode* mdCounter = MDNode::get(context, v3);  
-        counter++;
-        ii->setMetadata("wi", md);
-        ii->setMetadata("wi_counter", mdCounter);
-      }
-    }
-}
-
-
 /**
  * Inserts a new basic block to the region, before an old basic block in
  * the region.
@@ -604,70 +554,63 @@ ParallelRegion::AddBlockAfter(llvm::BasicBlock *block, llvm::BasicBlock *after)
     insert(afterPos, block);
 }
 
-bool 
-ParallelRegion::HasBlock(llvm::BasicBlock *bb)
-{
-    return find(begin(), end(), bb) != end();
+bool ParallelRegion::hasBlock(llvm::BasicBlock *Block) {
+  return find(begin(), end(), Block) != end();
 }
 
-/**
- * Find the instruction that loads the Z dimension of the work item
- * in the beginning of the parallel region, if not found, creates it.
- */
-llvm::Instruction*
-ParallelRegion::LocalIDZLoad()
-{
-  if (LocalIDZLoadInstr != NULL) return LocalIDZLoadInstr;
-  IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
-  GlobalVariable *Ptr = entryBB()->getParent()->getParent()->getGlobalVariable(
-      POCL_LOCAL_ID_Z_GLOBAL);
-  return LocalIDZLoadInstr = builder.CreateLoad(
-#if LLVM_MAJOR > 14
-             Ptr->getValueType(),
-#else
-             Ptr->getType()->getPointerElementType(),
-#endif
-             Ptr);
-}
+/// Finds the instruction that loads an id of the work item in the
+/// beginning of the parallel region, if not found, creates it.
+///
+/// \param IDGlobalName The name of the (magic) GlobalVariable temporally
+/// representing the id.
+/// \param Before If given, finds one in the basic block of the given
+/// instruction, or creates one just before it.
+/// \returns The instruction loading the id.
+llvm::Instruction *
+ParallelRegion::getOrCreateIDLoad(std::string IDGlobalName,
+                                  llvm::Instruction *Before) {
 
-/**
- * Find the instruction that loads the Y dimension of the work item
- * in the beginning of the parallel region, if not found, creates it.
- */
-llvm::Instruction*
-ParallelRegion::LocalIDYLoad()
-{
-  if (LocalIDYLoadInstr != NULL) return LocalIDYLoadInstr;
-  IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
-  GlobalVariable *Ptr = entryBB()->getParent()->getParent()->getGlobalVariable(
-      POCL_LOCAL_ID_Y_GLOBAL);
-  return LocalIDYLoadInstr = builder.CreateLoad(
-#if LLVM_MAJOR > 14
-             Ptr->getValueType(),
-#else
-             Ptr->getType()->getPointerElementType(),
-#endif
-             Ptr);
-}
+  Module *M = entryBB()->getParent()->getParent();
 
-/**
- * Find the instruction that loads the X dimension of the work item
- * in the beginning of the parallel region, if not found, creates it.
- */
-llvm::Instruction*
-ParallelRegion::LocalIDXLoad()
-{
-  if (LocalIDXLoadInstr != NULL) return LocalIDXLoadInstr;
-  IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
-  GlobalVariable *Ptr = entryBB()->getParent()->getParent()->getGlobalVariable(
-      POCL_LOCAL_ID_X_GLOBAL);
-  return LocalIDXLoadInstr = builder.CreateLoad(
-#if LLVM_MAJOR > 14
-             Ptr->getValueType(),
-#else
-             Ptr->getType()->getPointerElementType(),
-#endif
-             Ptr);
+  llvm::Type *ST = SizeT(M);
+  GlobalVariable *IDGlobal =
+      cast<GlobalVariable>(M->getOrInsertGlobal(IDGlobalName, ST));
+
+  if (Before != nullptr) {
+    // Try to find one in the same BB.
+    BasicBlock *BB = Before->getParent();
+    for (auto &I : *BB) {
+      Instruction *BBInst = &I;
+
+      if (BBInst == Before) {
+        // Didn't find one before it. Create one.
+        IRBuilder<> Builder(Before);
+        return Builder.CreateLoad(ST, IDGlobal);
+      }
+
+      LoadInst *Load = dyn_cast<LoadInst>(BBInst);
+      if (Load == nullptr)
+        continue;
+      GlobalVariable *Global =
+          dyn_cast<GlobalVariable>(Load->getPointerOperand());
+      if (Global == IDGlobal)
+        return Load;
+    }
+  }
+
+  // Otherwise, create one to the parallel region entry.
+  if (IDLoadInstrs.find(IDGlobalName) != IDLoadInstrs.end())
+    return IDLoadInstrs[IDGlobalName];
+
+  GlobalVariable *Ptr =
+      cast<GlobalVariable>(M->getOrInsertGlobal(IDGlobalName, ST));
+
+  llvm::BasicBlock &BB = *entryBB();
+  CreateBuilder(Builder, BB);
+
+  Instruction *IDLoad = Builder.CreateLoad(ST, IDGlobal);
+  IDLoadInstrs[IDGlobalName] = IDLoad;
+  return IDLoad;
 }
 
 void
@@ -725,7 +668,7 @@ ParallelRegion::InjectPrintF
   args.push_back(const_ptr_8);
   args.insert(args.end(), params.begin(), params.end());
 
-  CallInst::Create(printfFunc, args, "", before);
+  CallInst::Create(printfFunc, args, "", Inst2InsertPt(before));
 }
 
 void
@@ -767,103 +710,85 @@ ParallelRegion::InjectRegionPrintF()
   ConstantInt* pRID = ConstantInt::get(M->getContext(), APInt(32, pRegionId, 10));
   std::vector<Value*> params;
   params.push_back(pRID);
-  params.push_back(LocalIDXLoad());
-  params.push_back(LocalIDYLoad());
-  params.push_back(LocalIDZLoad());
+  params.push_back(getOrCreateIDLoad(LID_G_NAME(0)));
+  params.push_back(getOrCreateIDLoad(LID_G_NAME(1)));
+  params.push_back(getOrCreateIDLoad(LID_G_NAME(2)));
 
   InjectPrintF(exitBB()->getTerminator(), "PR %d WI %u %u %u\n", params);
 
 }
 
-/**
- * Adds a printf to the end of the parallel region that prints the
- * hex contents of all named non-pointer variables.
- *
- * Useful for debugging data flow bugs.
- */
-void
-ParallelRegion::InjectVariablePrintouts()
-{
-  for (ParallelRegion::iterator i = begin();
-       i != end(); ++i)
-    {
-      llvm::BasicBlock *bb = *i;
-      for (llvm::BasicBlock::iterator instr = bb->begin();
-           instr != bb->end(); ++instr) 
-        {
-          llvm::Instruction *instruction = &*instr;
-          if (isa<PointerType>(instruction->getType()) ||
-              !instruction->hasName()) continue;
-          std::string name = instruction->getName().str();
-          std::vector<Value*> args;
-          IRBuilder<> builder(exitBB()->getTerminator());
-          args.push_back(builder.CreateGlobalString(name));
-          args.push_back(instruction);
-          InjectPrintF(instruction->getParent()->getTerminator(), "variable %s == %x\n", args);
-        }
+/// Adds a printf to the end of the parallel region that prints the
+/// hex contents of all named non-pointer variables.
+///
+/// Useful for debugging data flow bugs.
+///
+void ParallelRegion::InjectVariablePrintouts() {
+  for (ParallelRegion::iterator i = begin(); i != end(); ++i) {
+    llvm::BasicBlock *BB = *i;
+    for (llvm::BasicBlock::iterator Instr = BB->begin(); Instr != BB->end();
+         ++Instr) {
+      llvm::Instruction *Instruction = &*Instr;
+      if (isa<PointerType>(Instruction->getType()) || !Instruction->hasName())
+        continue;
+      std::string Name = Instruction->getName().str();
+      std::vector<Value *> Args;
+      IRBuilder<> Builder(exitBB()->getTerminator());
+      Args.push_back(Builder.CreateGlobalString(Name));
+      Args.push_back(Instruction);
+      InjectPrintF(Instruction->getParent()->getTerminator(),
+                   "variable %s == %x\n", Args);
     }
+  }
 }
 
-/**
- * Localizes all the loads to the the work-item identifiers.
- *
- * In case the code inside the region queries the WI id, it
- * should not (re)use one that is loaded in another region, but
- * one that is loaded in the same region. Otherwise, it ends
- * up using the last id the previous PR work-item loop got.
- * This caused problems in cases where the local id was stored
- * to a temporary variable in an earlier region and that temp
- * was reused later.
- *
- * The function scans for all loads from the local id variables
- * and converts them to loads inside the parallel region.
- */
-void
-ParallelRegion::LocalizeIDLoads() 
-{
-  /* The local id loads inside the parallel region. */
-  llvm::Instruction* LocalIDXLoadInstr = LocalIDXLoad();
-  llvm::Instruction* LocalIDYLoadInstr = LocalIDYLoad();
-  llvm::Instruction* LocalIDZLoadInstr = LocalIDZLoad();
-  llvm::Module *M = LocalIDXLoadInstr->getParent()->getParent()->getParent();
-  llvm::Value *localIdZ = M->getNamedGlobal(POCL_LOCAL_ID_Z_GLOBAL);
-  llvm::Value *localIdY = M->getNamedGlobal(POCL_LOCAL_ID_Y_GLOBAL);
-  llvm::Value *localIdX = M->getNamedGlobal(POCL_LOCAL_ID_X_GLOBAL);
+/// Localizes all the loads to the the work-item identifiers.
+///
+/// In case the code inside the region queries the WI id, it should not (re)use
+/// one that is loaded in another region, but one that is loaded in the same
+/// region. Otherwise, it ends up using the last id the previous PR work-item
+/// loop got. This caused problems in cases where the local id was stored to a
+/// temporary variable in an earlier region and that temp was reused later.
+///
+/// The function scans for all accesses to the local and global ids and converts
+/// them to loads inside the parallel region.
+void ParallelRegion::localizeIDLoads() {
+  // The id loads inside the parallel region.
+  std::array<llvm::Instruction *, 6> RegionIDLoads = {
+      getOrCreateIDLoad(LID_G_NAME(0)), getOrCreateIDLoad(LID_G_NAME(1)),
+      getOrCreateIDLoad(LID_G_NAME(2)), getOrCreateIDLoad(GID_G_NAME(0)),
+      getOrCreateIDLoad(GID_G_NAME(1)), getOrCreateIDLoad(GID_G_NAME(2))};
 
-  assert (localIdZ != NULL && localIdY != NULL && localIdX != NULL &&
-	  "The local id globals were not created.");
+  llvm::Module *M = RegionIDLoads[0]->getParent()->getParent()->getParent();
 
-  for (ParallelRegion::iterator i = begin();
-       i != end(); ++i)
-    {
-      llvm::BasicBlock *bb = *i;
-      for (llvm::BasicBlock::iterator instrI = bb->begin();
-           instrI != bb->end(); ++instrI) 
-        {
-          llvm::Instruction *instr = &*instrI;
-	  if (instr == LocalIDXLoadInstr ||
-	      instr == LocalIDYLoadInstr ||
-	      instr == LocalIDZLoadInstr) continue;
+  std::array<llvm::Value *, 6> Globals = {
+      M->getNamedGlobal(LID_G_NAME(0)), M->getNamedGlobal(LID_G_NAME(1)),
+      M->getNamedGlobal(LID_G_NAME(2)), M->getNamedGlobal(GID_G_NAME(0)),
+      M->getNamedGlobal(GID_G_NAME(1)), M->getNamedGlobal(GID_G_NAME(2))};
 
-	  /* Search all operands of the instruction. If any of them is
-	     using a local id, replace it with the intra-PR load from the
-	     id variable. */
-          for (unsigned opr = 0; opr < instr->getNumOperands(); ++opr)
-            {
-	      llvm::LoadInst *load = 
-		dyn_cast<llvm::LoadInst>(instr->getOperand(opr));
-	      if (load == NULL) continue;
-	      if (load == LocalIDXLoadInstr ||
-		  load == LocalIDYLoadInstr ||
-		  load == LocalIDZLoadInstr) continue;
-	      
-	      if (load->getPointerOperand() == localIdZ)
-		instr->setOperand(opr, LocalIDZLoadInstr);
-	      if (load->getPointerOperand() == localIdY)
-		instr->setOperand(opr, LocalIDYLoadInstr);
-	      if (load->getPointerOperand() == localIdX)
-		instr->setOperand(opr, LocalIDXLoadInstr);
-	    }
-	}
+  for (ParallelRegion::iterator BBI = begin(); BBI != end(); ++BBI) {
+    llvm::BasicBlock *BB = *BBI;
+    for (llvm::BasicBlock::iterator II = BB->begin(); II != BB->end(); ++II) {
+      llvm::Instruction *Instr = &*II;
+
+      // If any of the operands is using an id, replace it with the
+      // intra-PR load from the parallel region specific id variable.
+      for (unsigned Opr = 0; Opr < Instr->getNumOperands(); ++Opr) {
+        llvm::LoadInst *Load = dyn_cast<llvm::LoadInst>(Instr->getOperand(Opr));
+        if (Load == NULL)
+          continue;
+
+        if (std::find(RegionIDLoads.begin(), RegionIDLoads.end(), Load) !=
+            RegionIDLoads.end())
+          continue; // Already converted.
+
+        auto Pos = std::find(Globals.begin(), Globals.end(),
+                             Load->getPointerOperand());
+        if (Pos == Globals.end())
+          continue;
+
+        Instr->setOperand(Opr, RegionIDLoads[Pos - Globals.begin()]);
+      }
     }
+  }
 }
