@@ -33,7 +33,6 @@ IGNORE_COMPILER_WARNING("-Wmaybe-uninitialized")
 #include <llvm/ADT/Twine.h>
 POP_COMPILER_DIAGS
 IGNORE_COMPILER_WARNING("-Wunused-parameter")
-#include <llvm/Analysis/PostDominators.h>
 #include <llvm/IR/DIBuilder.h>
 #include <llvm/IR/DebugInfo.h>
 #include <llvm/IR/Dominators.h>
@@ -49,10 +48,9 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 POP_COMPILER_DIAGS
 
 #include "Barrier.h"
+#include "KernelCompilerUtils.h"
 #include "LLVMUtils.h"
 #include "SubCFGFormation.h"
-#include "VariableUniformityAnalysis.h"
-#include "VariableUniformityAnalysisResult.hh"
 #include "Workgroup.h"
 #include "WorkitemHandlerChooser.h"
 
@@ -60,6 +58,7 @@ POP_COMPILER_DIAGS
 
 #include <cstddef>
 #include <functional>
+#include <iostream>
 #include <numeric>
 
 // #define DEBUG_SUBCFG_FORMATION
@@ -78,12 +77,6 @@ constexpr size_t ExitBarrierId = -1;
 static size_t DefaultAlignment = 64;
 static constexpr const char LoopStateMD[] = "poclLoopState";
 
-namespace PoclMDKind {
-  static constexpr const char Arrayified[] = "pocl.arrayified";
-  static constexpr const char InnerLoop[] = "pocl.loop.inner";
-  static constexpr const char WorkItemLoop[] = "pocl.loop.workitem";
-};
-
 static constexpr const char LocalIdGlobalNameX[] = "_local_id_x";
 static constexpr const char LocalIdGlobalNameY[] = "_local_id_y";
 static constexpr const char LocalIdGlobalNameZ[] = "_local_id_z";
@@ -94,11 +87,7 @@ static const std::array<char, 3> DimName{'x', 'y', 'z'};
 
 llvm::Loop *updateDtAndLi(llvm::LoopInfo &LI, llvm::DominatorTree &DT,
                           const llvm::BasicBlock *B, llvm::Function &F) {
-#if LLVM_VERSION_MAJOR < 11
-  DT.releaseMemory();
-#else
   DT.reset();
-#endif
   DT.recalculate(F);
   LI.releaseMemory();
   LI.analyze(DT);
@@ -122,7 +111,7 @@ bool anyOfUsers(llvm::Value *V, Func &&L) {
 // value to the alloca element at \a Idx.
 llvm::AllocaInst *arrayifyValue(llvm::Instruction *IPAllocas,
                                 llvm::Value *ToArrayify,
-                                llvm::Instruction *InsertionPoint,
+                                BasicBlock::iterator InsertionPoint,
                                 llvm::Value *Idx, llvm::Value *NumElements,
                                 llvm::MDTuple *MDAlloca = nullptr) {
   assert(Idx && "Valid WI-Index required");
@@ -138,15 +127,17 @@ llvm::AllocaInst *arrayifyValue(llvm::Instruction *IPAllocas,
                                             ToArrayify->getName() + "_alloca");
   if (NumElements)
     Alloca->setAlignment(llvm::Align{DefaultAlignment});
-  Alloca->setMetadata(PoclMDKind::Arrayified, MDAlloca);
+  Alloca->setMetadata(PoCLMDKind::Arrayified, MDAlloca);
+
+  IRBuilder<> WriteBuilder{AllocaBuilder.getContext()};
+  WriteBuilder.SetInsertPoint(InsertionPoint->getParent(), InsertionPoint);
 
-  llvm::IRBuilder<> WriteBuilder{InsertionPoint};
   llvm::Value *StoreTarget = Alloca;
   if (NumElements) {
     auto *GEP = llvm::cast<llvm::GetElementPtrInst>(
         WriteBuilder.CreateInBoundsGEP(Alloca->getAllocatedType(), Alloca, Idx,
                                        ToArrayify->getName() + "_gep"));
-    GEP->setMetadata(PoclMDKind::Arrayified, MDAlloca);
+    GEP->setMetadata(PoCLMDKind::Arrayified, MDAlloca);
     StoreTarget = GEP;
   }
   WriteBuilder.CreateStore(ToArrayify, StoreTarget);
@@ -159,9 +150,9 @@ llvm::AllocaInst *arrayifyInstruction(llvm::Instruction *IPAllocas,
                                       llvm::Value *Idx,
                                       llvm::Value *NumElements,
                                       llvm::MDTuple *MDAlloca = nullptr) {
-  llvm::Instruction *InsertionPoint = &*(++ToArrayify->getIterator());
+  BasicBlock::iterator InsertionPoint = ++ToArrayify->getIterator();
   if (llvm::isa<llvm::PHINode>(ToArrayify))
-    InsertionPoint = ToArrayify->getParent()->getFirstNonPHI();
+    InsertionPoint = ToArrayify->getParent()->getFirstInsertionPt();
 
   return arrayifyValue(IPAllocas, ToArrayify, InsertionPoint, Idx, NumElements,
                        MDAlloca);
@@ -173,7 +164,7 @@ llvm::LoadInst *loadFromAlloca(llvm::AllocaInst *Alloca, llvm::Value *Idx,
                                llvm::Instruction *InsertBefore,
                                const llvm::Twine &NamePrefix = "") {
   assert(Idx && "Valid WI-Index required");
-  auto *MDAlloca = Alloca->getMetadata(PoclMDKind::Arrayified);
+  auto *MDAlloca = Alloca->getMetadata(PoCLMDKind::Arrayified);
 
   llvm::IRBuilder<> LoadBuilder{InsertBefore};
   llvm::Value *LoadFrom = Alloca;
@@ -181,7 +172,7 @@ llvm::LoadInst *loadFromAlloca(llvm::AllocaInst *Alloca, llvm::Value *Idx,
     auto *GEP =
         llvm::cast<llvm::GetElementPtrInst>(LoadBuilder.CreateInBoundsGEP(
             Alloca->getAllocatedType(), Alloca, Idx, NamePrefix + "_lgep"));
-    GEP->setMetadata(PoclMDKind::Arrayified, MDAlloca);
+    GEP->setMetadata(PoCLMDKind::Arrayified, MDAlloca);
     LoadFrom = GEP;
   }
   auto *Load = LoadBuilder.CreateLoad(Alloca->getAllocatedType(), LoadFrom,
@@ -198,7 +189,7 @@ llvm::AllocaInst *getLoopStateAllocaForLoad(llvm::LoadInst &LInst) {
   } else {
     Alloca = llvm::dyn_cast<llvm::AllocaInst>(LInst.getPointerOperand());
   }
-  if (Alloca && Alloca->hasMetadata(PoclMDKind::Arrayified))
+  if (Alloca && Alloca->hasMetadata(PoCLMDKind::Arrayified))
     return Alloca;
   return nullptr;
 }
@@ -278,10 +269,17 @@ getLocalSizeValues(llvm::Function &F, llvm::ArrayRef<unsigned long> LocalSizes,
 
       if (I->getParent() != &F.getEntryBlock()) {
         // must be in entry block. move.
+// TODO for some reason, moveAfter(InsertionPoint) exists in the header, but
+// linking fails with undefined symbol
+//#if LLVM_MAJOR < 20
+          auto InsPt = F.getEntryBlock().getFirstNonPHI();
+//#else
+//          BasicBlock::iterator InsPt = F.getEntryBlock().getFirstInsertionPt();
+//#endif
         if (F.getEntryBlock().size() == 1)
-          I->moveBefore(F.getEntryBlock().getFirstNonPHI());
+          I->moveBefore(InsPt);
         else
-          I->moveAfter(F.getEntryBlock().getFirstNonPHI());
+          I->moveAfter(InsPt);
       }
     } else
       LocalSize[D] = llvm::ConstantInt::get(
@@ -324,6 +322,12 @@ void createLoopsAround(llvm::Function &F, llvm::BasicBlock *AfterBB,
         Builder.getIntN(DL.getLargestLegalIntTypeSizeInBits(), 0),
         &F.getEntryBlock());
     IndVars.push_back(WIIndVar);
+
+    // Create the global id calculations.
+    llvm::GlobalVariable *LocalIDG =
+        F.getParent()->getGlobalVariable(LID_G_NAME(D));
+    assert(LocalIDG != nullptr);
+    Builder.CreateStore(WIIndVar, LocalIDG);
     Builder.CreateBr(LastHeader);
 
     auto *Latch =
@@ -349,7 +353,7 @@ void createLoopsAround(llvm::Function &F, llvm::BasicBlock *AfterBB,
 
   auto *MDWorkItemLoop = llvm::MDNode::get(
       F.getContext(),
-      {llvm::MDString::get(F.getContext(), PoclMDKind::WorkItemLoop)});
+      {llvm::MDString::get(F.getContext(), PoCLMDKind::WorkItemLoop)});
   auto *LoopID = llvm::makePostTransformationMetadata(F.getContext(), nullptr,
                                                       {}, {MDWorkItemLoop});
   Latches[0]->getTerminator()->setMetadata("llvm.loop", LoopID);
@@ -378,105 +382,6 @@ void createLoopsAround(llvm::Function &F, llvm::BasicBlock *AfterBB,
   ContiguousIdx = Idx;
 }
 
-class SubCFG {
-
-  using BlockVector = llvm::SmallVector<llvm::BasicBlock *, 8>;
-  BlockVector Blocks_;
-  BlockVector NewBlocks_;
-  size_t EntryId_;
-  llvm::BasicBlock *EntryBarrier_;
-  llvm::SmallDenseMap<llvm::BasicBlock *, size_t> ExitIds_;
-  llvm::AllocaInst *LastBarrierIdStorage_;
-  llvm::Value *ContIdx_;
-  llvm::BasicBlock *EntryBB_;
-  llvm::BasicBlock *ExitBB_;
-  llvm::BasicBlock *LoadBB_;
-  llvm::BasicBlock *PreHeader_;
-  size_t Dim;
-
-  llvm::BasicBlock *createExitWithID(
-      llvm::detail::DenseMapPair<llvm::BasicBlock *, size_t> BarrierPair,
-      llvm::BasicBlock *After, llvm::BasicBlock *TargetBB);
-
-  void loadMultiSubCfgValues(
-      const llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-          &InstAllocaMap,
-      llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-          &BaseInstAllocaMap,
-      llvm::DenseMap<llvm::Instruction *,
-                     llvm::SmallVector<llvm::Instruction *, 8>>
-          &ContInstReplicaMap,
-      llvm::BasicBlock *UniformLoadBB, llvm::ValueToValueMapTy &VMap);
-  void loadUniformAndRecalcContValues(
-      llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-          &BaseInstAllocaMap,
-      llvm::DenseMap<llvm::Instruction *,
-                     llvm::SmallVector<llvm::Instruction *, 8>>
-          &ContInstReplicaMap,
-      llvm::BasicBlock *UniformLoadBB, llvm::ValueToValueMapTy &VMap);
-  llvm::BasicBlock *createLoadBB(llvm::ValueToValueMapTy &VMap);
-  llvm::BasicBlock *createUniformLoadBB(llvm::BasicBlock *OuterMostHeader);
-
-public:
-  SubCFG(llvm::BasicBlock *EntryBarrier, llvm::AllocaInst *LastBarrierIdStorage,
-         const llvm::DenseMap<llvm::BasicBlock *, size_t> &BarrierIds,
-         llvm::Value *IndVar, size_t Dim);
-
-  SubCFG(const SubCFG &) = delete;
-  SubCFG &operator=(const SubCFG &) = delete;
-
-  SubCFG(SubCFG &&) = default;
-  SubCFG &operator=(SubCFG &&) = default;
-
-  BlockVector &getBlocks() noexcept { return Blocks_; }
-  const BlockVector &getBlocks() const noexcept { return Blocks_; }
-
-  BlockVector &getNewBlocks() noexcept { return NewBlocks_; }
-  const BlockVector &getNewBlocks() const noexcept { return NewBlocks_; }
-
-  size_t getEntryId() const noexcept { return EntryId_; }
-
-  llvm::BasicBlock *getEntry() noexcept { return EntryBB_; }
-  llvm::BasicBlock *getExit() noexcept { return ExitBB_; }
-  llvm::BasicBlock *getLoadBB() noexcept { return LoadBB_; }
-  llvm::Value *getContiguousIdx() noexcept { return ContIdx_; }
-
-  void replicate(llvm::Function &F,
-                 const llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-                     &InstAllocaMap,
-                 llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-                     &BaseInstAllocaMap,
-                 llvm::DenseMap<llvm::Instruction *,
-                                llvm::SmallVector<llvm::Instruction *, 8>>
-                     &ContInstReplicaMap,
-                 llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-                     &RemappedInstAllocaMap,
-                 llvm::BasicBlock *AfterBB,
-                 llvm::ArrayRef<llvm::Value *> LocalSize);
-
-  void arrayifyMultiSubCfgValues(
-      llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *> &InstAllocaMap,
-      llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-          &BaseInstAllocaMap,
-      llvm::DenseMap<llvm::Instruction *,
-                     llvm::SmallVector<llvm::Instruction *, 8>>
-          &ContInstReplicaMap,
-      llvm::ArrayRef<SubCFG> SubCFGs, llvm::Instruction *AllocaIP,
-      llvm::Value *ReqdArrayElements,
-      pocl::VariableUniformityAnalysisResult &VecInfo);
-  void fixSingleSubCfgValues(
-      llvm::DominatorTree &DT,
-      const llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *>
-          &RemappedInstAllocaMap,
-      llvm::Value *ReqdArrayElements,
-      pocl::VariableUniformityAnalysisResult &VecInfo);
-
-  void print() const;
-  void removeDeadPhiBlocks(
-      llvm::SmallVector<llvm::BasicBlock *, 8> &BlocksToRemap) const;
-  llvm::SmallVector<llvm::Instruction *, 16> topoSortInstructions(
-      const llvm::SmallPtrSet<llvm::Instruction *, 16> &UniquifyInsts) const;
-};
 
 // create new exiting block writing the exit's id to LastBarrierIdStorage_
 llvm::BasicBlock *SubCFG::createExitWithID(
@@ -759,7 +664,7 @@ void SubCFG::arrayifyMultiSubCfgValues(
           }
         // GEP from already widened alloca: reuse alloca
         if (auto *GEP = llvm::dyn_cast<llvm::GetElementPtrInst>(&I))
-          if (GEP->hasMetadata(PoclMDKind::Arrayified)) {
+          if (GEP->hasMetadata(PoCLMDKind::Arrayified)) {
             InstAllocaMap.insert(
                 {&I, llvm::cast<llvm::AllocaInst>(GEP->getPointerOperand())});
             continue;
@@ -842,12 +747,12 @@ void SubCFG::loadMultiSubCfgValues(
               })) {
         if (auto *GEP =
                 llvm::dyn_cast<llvm::GetElementPtrInst>(InstAllocaPair.first))
-          if (auto *MDArrayified = GEP->getMetadata(PoclMDKind::Arrayified)) {
+          if (auto *MDArrayified = GEP->getMetadata(PoCLMDKind::Arrayified)) {
             auto *NewGEP =
                 llvm::cast<llvm::GetElementPtrInst>(Builder.CreateInBoundsGEP(
                     GEP->getType(), GEP->getPointerOperand(), NewContIdx,
                     GEP->getName() + "c"));
-            NewGEP->setMetadata(PoclMDKind::Arrayified, MDArrayified);
+            NewGEP->setMetadata(PoCLMDKind::Arrayified, MDArrayified);
             VMap[InstAllocaPair.first] = NewGEP;
             continue;
           }
@@ -925,7 +830,11 @@ void SubCFG::loadUniformAndRecalcContValues(
                  << " to " << LoadTerm->getParent()->getName() << "\n";
 #endif
     auto *IClone = I->clone();
+#if LLVM_MAJOR < 20
     IClone->insertBefore(LoadTerm);
+#else
+    IClone->insertBefore(LoadTerm->getIterator());
+#endif
     InstsToRemap.insert(IClone);
     UniVMap[I] = IClone;
     if (VMap.count(I) == 0)
@@ -1053,12 +962,12 @@ void SubCFG::fixSingleSubCfgValues(
 
           if (auto *GEP = llvm::dyn_cast<llvm::GetElementPtrInst>(OPI))
             if (auto *MDArrayified =
-                    GEP->getMetadata(PoclMDKind::Arrayified)) {
+                    GEP->getMetadata(PoCLMDKind::Arrayified)) {
               auto *NewGEP =
                   llvm::cast<llvm::GetElementPtrInst>(Builder.CreateInBoundsGEP(
                       GEP->getType(), GEP->getPointerOperand(), ContIdx_,
                       GEP->getName() + "c"));
-              NewGEP->setMetadata(PoclMDKind::Arrayified, MDArrayified);
+              NewGEP->setMetadata(PoCLMDKind::Arrayified, MDArrayified);
               I.replaceUsesOfWith(OPI, NewGEP);
               InstLoadMap.insert({OPI, NewGEP});
               continue;
@@ -1229,7 +1138,8 @@ template <class PtrSet> struct PtrSetWrapper {
 };
 
 // checks if all uses of an alloca are in just a single subcfg (doesn't have to
-// be arrayified!)
+// be arrayified!).
+// TO CLEAN: merge with WorkitemLoopsImpl::shouldNotBeContextSaved().
 bool isAllocaSubCfgInternal(llvm::AllocaInst *Alloca,
                             const std::vector<SubCFG> &SubCfgs,
                             const llvm::DominatorTree &DT) {
@@ -1271,12 +1181,13 @@ bool isAllocaSubCfgInternal(llvm::AllocaInst *Alloca,
 }
 
 // Widens the allocas in the entry block to array allocas.
-// Replace uses of the original alloca with GEP that indexes the new alloca with
+// Replace uses of the original alloca with a GEP that indexes the new alloca
+// with
 // \a Idx.
-void arrayifyAllocas(llvm::BasicBlock *EntryBlock, llvm::DominatorTree &DT,
-                     std::vector<SubCFG> &SubCfgs,
-                     llvm::Value *ReqdArrayElements,
-                     pocl::VariableUniformityAnalysisResult &VecInfo) {
+void SubCFGFormation::arrayifyAllocas(
+    llvm::BasicBlock *EntryBlock, llvm::DominatorTree &DT,
+    std::vector<SubCFG> &SubCfgs, llvm::Value *ReqdArrayElements,
+    pocl::VariableUniformityAnalysisResult &VecInfo) {
   auto *MDAlloca = llvm::MDNode::get(
       EntryBlock->getContext(),
       {llvm::MDString::get(EntryBlock->getContext(), "poclLoopState")});
@@ -1289,7 +1200,7 @@ void arrayifyAllocas(llvm::BasicBlock *EntryBlock, llvm::DominatorTree &DT,
   llvm::SmallVector<llvm::AllocaInst *, 8> WL;
   for (auto &I : *EntryBlock) {
     if (auto *Alloca = llvm::dyn_cast<llvm::AllocaInst>(&I)) {
-      if (Alloca->hasMetadata(PoclMDKind::Arrayified))
+      if (Alloca->hasMetadata(PoCLMDKind::Arrayified))
         continue; // already arrayified
       if (anyOfUsers<llvm::Instruction>(Alloca, [&SubCfgsBlocks](
                                                     llvm::Instruction *UI) {
@@ -1301,31 +1212,23 @@ void arrayifyAllocas(llvm::BasicBlock *EntryBlock, llvm::DominatorTree &DT,
     }
   }
 
+  BasicBlock &Entry = K->getEntryBlock();
   for (auto *I : WL) {
-    llvm::IRBuilder<> AllocaBuilder{I};
-    llvm::Type *T = I->getAllocatedType();
-    if (auto *ArrSizeC = llvm::dyn_cast<llvm::ConstantInt>(I->getArraySize())) {
-      auto ArrSize = ArrSizeC->getLimitedValue();
-      if (ArrSize > 1) {
-        T = llvm::ArrayType::get(T, ArrSize);
-        llvm::errs() << "Caution, alloca was array\n";
-      }
-    }
-
-    auto *Alloca = AllocaBuilder.CreateAlloca(T, ReqdArrayElements,
-                                              I->getName() + "_alloca");
-    Alloca->setAlignment(llvm::Align{DefaultAlignment});
-    Alloca->setMetadata(PoclMDKind::Arrayified, MDAlloca);
+    bool PaddingAdded = false;
+    llvm::AllocaInst *Alloca = createAlignedAndPaddedContextAlloca(
+        I, &*Entry.getFirstInsertionPt(), std::string(I->getName()) + "_alloca",
+        PaddingAdded);
+    Alloca->setMetadata(PoCLMDKind::Arrayified, MDAlloca);
 
     for (auto &SubCfg : SubCfgs) {
-      auto *GepIp = SubCfg.getLoadBB()->getFirstNonPHIOrDbgOrLifetime();
 
-      llvm::IRBuilder<> LoadBuilder{GepIp};
-      auto *GEP =
-          llvm::cast<llvm::GetElementPtrInst>(LoadBuilder.CreateInBoundsGEP(
-              Alloca->getAllocatedType(), Alloca, SubCfg.getContiguousIdx(),
-              I->getName() + "_gep"));
-      GEP->setMetadata(PoclMDKind::Arrayified, MDAlloca);
+      auto Before = SubCfg.getLoadBB()->getFirstNonPHIOrDbgOrLifetime();
+#if LLVM_MAJOR < 20
+      auto GEP = createContextArrayGEP(Alloca, Before, PaddingAdded);
+#else
+      auto GEP = createContextArrayGEP(Alloca, &*Before, PaddingAdded);
+#endif
+      GEP->setMetadata(PoCLMDKind::Arrayified, MDAlloca);
 
       llvm::replaceDominatedUsesWith(I, GEP, DT, SubCfg.getLoadBB());
     }
@@ -1333,6 +1236,47 @@ void arrayifyAllocas(llvm::BasicBlock *EntryBlock, llvm::DominatorTree &DT,
   }
 }
 
+llvm::Value *
+SubCFGFormation::getLinearWIIndexInRegion(llvm::Instruction *Instr) {
+  SubCFG *Region = regionOfBlock(Instr->getParent());
+  assert(Region != nullptr);
+  return Region->getContiguousIdx();
+}
+
+llvm::Instruction *SubCFGFormation::getLocalIdInRegion(llvm::Instruction *Instr,
+                                                       size_t Dim) {
+  SubCFG *Region = regionOfBlock(Instr->getParent());
+
+  std::string VarName = LID_G_NAME(Dim);
+  BasicBlock *LoadBB =
+      Region != nullptr ? Region->getLoadBB() : Instr->getParent();
+  auto *GV = K->getParent()->getOrInsertGlobal(VarName, ST);
+
+  for (auto &I : *LoadBB) {
+    // Ensure the local id load is defined before the referred instruction.
+    if (&I == Instr)
+      break;
+    if (auto *LoadI = llvm::dyn_cast<llvm::LoadInst>(&I)) {
+      if (LoadI->getPointerOperand() == GV)
+        return &I;
+    }
+  }
+#if LLVM_MAJOR < 20
+  llvm::IRBuilder<> Builder(LoadBB->getFirstNonPHI());
+#else
+  IRBuilder<> Builder{LoadBB->getContext()};
+  Builder.SetInsertPoint(LoadBB->getFirstInsertionPt());
+#endif
+
+  return Builder.CreateLoad(ST, GV);
+}
+
+/// CBS does not handle kernels without barriers.
+bool SubCFGFormation::canHandleKernel(llvm::Function &K,
+                                      llvm::FunctionAnalysisManager &AM) {
+  return hasWorkgroupBarriers(K);
+}
+
 void moveAllocasToEntry(llvm::Function &F,
                         llvm::ArrayRef<llvm::BasicBlock *> Blocks) {
   llvm::SmallVector<llvm::AllocaInst *, 4> AllocaWL;
@@ -1341,7 +1285,11 @@ void moveAllocasToEntry(llvm::Function &F,
       if (auto *AllocaInst = llvm::dyn_cast<llvm::AllocaInst>(&I))
         AllocaWL.push_back(AllocaInst);
   for (auto *I : AllocaWL)
+#if LLVM_MAJOR < 20
     I->moveBefore(F.getEntryBlock().getTerminator());
+#else
+    I->moveBefore(F.getEntryBlock().getTerminator()->getIterator());
+#endif
 }
 
 llvm::DenseMap<llvm::BasicBlock *, size_t>
@@ -1362,9 +1310,11 @@ getBarrierIds(llvm::BasicBlock *Entry,
       Barriers.insert({BB, BarrierId++});
   return Barriers;
 }
-void formSubCfgs(llvm::Function &F, llvm::LoopInfo &LI, llvm::DominatorTree &DT,
-                 llvm::PostDominatorTree &PDT,
-                 pocl::VariableUniformityAnalysisResult &VUA) {
+
+void SubCFGFormation::formSubCfgs(llvm::Function &F, llvm::LoopInfo &LI,
+                                  llvm::DominatorTree &DT,
+                                  llvm::PostDominatorTree &PDT,
+                                  pocl::VariableUniformityAnalysisResult &VUA) {
 #ifdef DEBUG_SUBCFG_FORMATION
   F.viewCFG();
 #endif
@@ -1406,6 +1356,8 @@ void formSubCfgs(llvm::Function &F, llvm::LoopInfo &LI, llvm::DominatorTree &DT,
                  [](auto &BB) { return &BB; });
 
   // non-entry block Allocas are considered broken, move to entry.
+  // TODO: Unify with the AllocasToEntry pass. Perhaps convert to
+  // a WorkitemHandler helper function.
   moveAllocasToEntry(F, Blocks);
 
 // kept for simple reenabling of more advanced uniformity analysis
@@ -1443,8 +1395,7 @@ void formSubCfgs(llvm::Function &F, llvm::LoopInfo &LI, llvm::DominatorTree &DT,
   VecInfo.setPinnedShape(*IndVar, pocl::VectorShape::cont());
 #endif
 
-  // create subcfgs
-  std::vector<SubCFG> SubCFGs;
+  SubCFGs.clear();
   for (auto &BIt : Barriers) {
 #ifdef DEBUG_SUBCFG_FORMATION
     llvm::errs() << "Create SubCFG from " << BIt.first->getName() << "("
@@ -1465,18 +1416,23 @@ void formSubCfgs(llvm::Function &F, llvm::LoopInfo &LI, llvm::DominatorTree &DT,
         InstAllocaMap, BaseInstAllocaMap, InstContReplicaMap, SubCFGs,
         F.getEntryBlock().getTerminator(), ReqdArrayElements, VUA);
 
+  llvm::BasicBlock *NewExit =
+      llvm::BasicBlock::Create(F.getContext(), "cbs.exit", &F);
+  llvm::IRBuilder<> ExitBuilder(NewExit);
+  ExitBuilder.CreateRetVoid();
+
   llvm::DenseMap<llvm::Instruction *, llvm::AllocaInst *> RemappedInstAllocaMap;
   for (auto &Cfg : SubCFGs) {
     Cfg.print();
     Cfg.replicate(F, InstAllocaMap, BaseInstAllocaMap, InstContReplicaMap,
-                  RemappedInstAllocaMap, *ExitingBlocks.begin(), LocalSize);
+                  RemappedInstAllocaMap, NewExit, LocalSize);
     purgeLifetime(Cfg);
   }
 
   llvm::BasicBlock *WhileHeader = nullptr;
   WhileHeader = generateWhileSwitchAround(
-      &F.getEntryBlock(), F.getEntryBlock().getSingleSuccessor(),
-      *ExitingBlocks.begin(), LastBarrierIdStorage, SubCFGs);
+      &F.getEntryBlock(), F.getEntryBlock().getSingleSuccessor(), NewExit,
+      LastBarrierIdStorage, SubCFGs);
 
   llvm::removeUnreachableBlocks(F);
 
@@ -1502,6 +1458,19 @@ void formSubCfgs(llvm::Function &F, llvm::LoopInfo &LI, llvm::DominatorTree &DT,
   llvm::simplifyLoop(WhileLoop, &DT, &LI, nullptr, nullptr, nullptr, false);
 }
 
+// Finds the SubCFG in the currently found SubCFGs which has the given BB.
+SubCFG *SubCFGFormation::regionOfBlock(llvm::BasicBlock *BB) {
+  for (auto &Region : SubCFGs) {
+    if (Region.getLoadBB() == BB)
+      return &Region;
+    SubCFG::BlockVector &BlocksInRegion = Region.getBlocks();
+    for (auto *Block : BlocksInRegion)
+      if (Block == BB)
+        return &Region;
+  }
+  return nullptr;
+}
+
 void createParallelAccessesMdOrAddAccessGroup(const llvm::Function *F,
                                               llvm::Loop *const &L,
                                               llvm::MDNode *MDAccessGroup) {
@@ -1543,9 +1512,6 @@ void addAccessGroupMD(llvm::Instruction *I, llvm::MDNode *MDAccessGroup) {
     I->setMetadata(llvm::LLVMContext::MD_access_group, MDAccessGroup);
 }
 
-#if LLVM_VERSION_MAJOR > 12 ||                                                 \
-    (LLVM_VERSION_MAJOR == 12 && LLVM_VERSION_MINOR == 0 &&                    \
-     LLVM_VERSION_PATCH == 1)
 void markLoopParallel(llvm::Function &F, llvm::Loop *L) {
   // LLVM < 12.0.1 might miscompile if conditionals in "parallel" loop
   // (https://llvm.org/PR46666)
@@ -1564,57 +1530,6 @@ void markLoopParallel(llvm::Function &F, llvm::Loop *L) {
   // make the access group parallel w.r.t the WI loop
   createParallelAccessesMdOrAddAccessGroup(&F, L, MDAccessGroup);
 }
-#else
-void markLoopParallel(llvm::Function &, llvm::Loop *) {}
-#endif
-
-#if LLVM_MAJOR < MIN_LLVM_NEW_PASSMANAGER
-char SubCFGFormation::ID = 0;
-
-void SubCFGFormation::getAnalysisUsage(llvm::AnalysisUsage &AU) const {
-  AU.addRequired<llvm::LoopInfoWrapperPass>();
-  AU.addRequiredTransitive<llvm::DominatorTreeWrapperPass>();
-  AU.addRequiredTransitive<llvm::PostDominatorTreeWrapperPass>();
-
-  AU.addRequired<VariableUniformityAnalysis>();
-  AU.addPreserved<VariableUniformityAnalysis>();
-
-  AU.addRequired<WorkitemHandlerChooser>();
-  AU.addPreserved<WorkitemHandlerChooser>();
-}
-
-bool SubCFGFormation::runOnFunction(llvm::Function &F) {
-  if (!isKernelToProcess(F))
-    return false;
-
-  auto WIH = getAnalysis<pocl::WorkitemHandlerChooser>().chosenHandler();
-  if (WIH != WorkitemHandlerType::CBS)
-    return false;
-  if (!hasWorkgroupBarriers(F))
-    return false;
-
-#ifdef DEBUG_SUBCFG_FORMATION
-  llvm::errs() << "[SubCFG] Form SubCFGs in " << F.getName() << "\n";
-#endif
-
-  auto &DT = getAnalysis<llvm::DominatorTreeWrapperPass>().getDomTree();
-  auto &PDT =
-      getAnalysis<llvm::PostDominatorTreeWrapperPass>().getPostDomTree();
-  auto &LI = getAnalysis<llvm::LoopInfoWrapperPass>().getLoopInfo();
-  auto &VUA = getAnalysis<pocl::VariableUniformityAnalysis>().getResult();
-
-  formSubCfgs(F, LI, DT, PDT, VUA);
-
-  for (auto *SL : LI.getLoopsInPreorder())
-    if (llvm::findOptionMDForLoop(SL, PoclMDKind::WorkItemLoop))
-      markLoopParallel(F, SL);
-
-  return true;
-}
-
-REGISTER_OLD_FPASS(PASS_NAME, PASS_CLASS, PASS_DESC);
-
-#else
 
 // enable new pass manager infrastructure
 llvm::PreservedAnalyses
@@ -1626,10 +1541,11 @@ SubCFGFormation::run(llvm::Function &F, llvm::FunctionAnalysisManager &AM) {
   if (WIH != WorkitemHandlerType::CBS)
     return PreservedAnalyses::all();
 
-  if (!hasWorkgroupBarriers(F))
-    return PreservedAnalyses::all();
+  Initialize(cast<pocl::Kernel>(&F));
 
+#ifdef DEBUG_SUBCFG_FORMATION
   llvm::errs() << "[SubCFG] Form SubCFGs in " << F.getName() << "\n";
+#endif
 
   auto &DT = AM.getResult<llvm::DominatorTreeAnalysis>(F);
   auto &PDT = AM.getResult<llvm::PostDominatorTreeAnalysis>(F);
@@ -1638,9 +1554,15 @@ SubCFGFormation::run(llvm::Function &F, llvm::FunctionAnalysisManager &AM) {
 
   formSubCfgs(F, LI, DT, PDT, VUA);
 
-  for (auto *SL : LI.getLoopsInPreorder())
-    if (llvm::findOptionMDForLoop(SL, PoclMDKind::WorkItemLoop))
-      markLoopParallel(F, SL);
+  if (canAnnotateParallelLoops())
+    for (auto *SL : LI.getLoopsInPreorder())
+      if (llvm::findOptionMDForLoop(SL, PoCLMDKind::WorkItemLoop))
+        markLoopParallel(F, SL);
+
+  handleLocalMemAllocas();
+
+  handleWorkitemFunctions();
+  GenerateGlobalIdComputation();
 
   PreservedAnalyses PAChanged = PreservedAnalyses::none();
   PAChanged.preserve<WorkitemHandlerChooser>();
@@ -1649,6 +1571,4 @@ SubCFGFormation::run(llvm::Function &F, llvm::FunctionAnalysisManager &AM) {
 
 REGISTER_NEW_FPASS(PASS_NAME, PASS_CLASS, PASS_DESC);
 
-#endif
-
 } // namespace pocl
