@@ -22,9 +22,10 @@
 */
 
 #include "rdma_request_th.hh"
+#include <infiniband/verbs.h>
 
 RdmaRequestThread::RdmaRequestThread(
-    VirtualContextBase *c, ExitHelper *e, TrafficMonitor *tm,
+    VirtualContextBase *c, ExitHelper *e, std::shared_ptr<TrafficMonitor> tm,
     const char *id_str, std::shared_ptr<RdmaConnection> conn,
     std::unordered_map<uint32_t, RdmaBufferData> *mem_regions,
     std::mutex *mem_regions_mutex)
@@ -35,6 +36,7 @@ RdmaRequestThread::RdmaRequestThread(
 
 RdmaRequestThread::~RdmaRequestThread() {
   eh->requestExit(id_str.c_str(), 0);
+  connection->disconnect();
   io_thread.join();
 }
 
@@ -84,6 +86,9 @@ void RdmaRequestThread::rdmaReaderThread() {
       break;
     }
 
+    if (wc.status != IBV_WC_SUCCESS)
+      break;
+
     if (netstat) {
       // The buffer contents are sent with IBV_WRITE and we have no way of
       // knowing the buffer size so just log the difference as requested here.
@@ -101,14 +106,14 @@ void RdmaRequestThread::rdmaReaderThread() {
 
     /************** Forward the Request to virtual context ******************/
     Request *request = new Request;
-    request->req = requests_buf.at(wc.wr_id);
+    request->Body = requests_buf.at(wc.wr_id);
 
     POCL_MSG_PRINT_GENERAL(
         "%s: received IBV_SEND for message ID: %" PRIu64 " type: %s\n",
-        id_str.c_str(), uint64_t(request->req.msg_id),
-        request_to_str((RequestMessageType)request->req.message_type));
+        id_str.c_str(), uint64_t(request->Body.msg_id),
+        request_to_str((RequestMessageType)request->Body.message_type));
 
-    switch (request->req.message_type) {
+    switch (request->Body.message_type) {
     case MessageType_ConnectPeer:
     case MessageType_DeviceInfo:
     case MessageType_CreateBuffer:
@@ -153,7 +158,7 @@ void RdmaRequestThread::rdmaReaderThread() {
       break;
     }
     case MessageType_NotifyEvent: {
-      virtualContext->notifyEvent(request->req.event_id, CL_SUCCESS);
+      virtualContext->notifyEvent(request->Body.event_id, CL_SUCCESS);
       delete request;
       break;
     }
