@@ -1,17 +1,17 @@
 /* Tests the crash with a barrier *between* two for-loops.
 
    Copyright (c) 2012 Pekka Jääskeläinen / Tampere University of Technology
-   
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -25,8 +25,7 @@
 
 // Enable OpenCL C++ exceptions
 #define CL_HPP_ENABLE_EXCEPTIONS
-#define CL_HPP_MINIMUM_OPENCL_VERSION 120
-#define CL_HPP_TARGET_OPENCL_VERSION 120
+
 #include <CL/opencl.hpp>
 
 #include <cstdio>
@@ -46,28 +45,32 @@
 // increasing the loop counter to 32 produces yet another crash
 
 static char
-kernelSourceCode[] = 
-"kernel \n"
-"void test_kernel(__global float *input, \n"
-"                 __global int *result) {\n"
-"  int gid = get_global_id(0);\n"
-"  float global_sum = 0.0f;\n"
-"  int i;\n"
-"\n"
-" result[gid] = global_sum;\n"
-" for (i=0; i < 32; ++i) {\n"
-"   float value = input[gid+i];\n"
-"   global_sum += value;\n"
-" }\n"
-" result[gid] = result[gid] + global_sum;\n"
-" barrier(CLK_GLOBAL_MEM_FENCE);\n"
-" \n"
-" for (i=0; i < 32; ++i) {\n"
-"   float value = input[gid+i];\n"
-"   global_sum += value;\n"
-" }\n"
-" result[gid] = result[gid] + global_sum;\n"
-"}\n";
+kernelSourceCode[] = R"CL(
+kernel
+void test_kernel(__global float *input,
+                 __global int *result) {
+  int gid = get_global_id(0);
+  float global_sum = 0.0f;
+  int i;
+
+  result[gid] = global_sum;
+  for (i=0; i < 32; ++i) {
+    float value = input[gid+i];
+    global_sum += value;
+  }
+  result[gid] = result[gid] + global_sum;
+//  printf("before barrier GID %d result[gid] == %f global_sum == %f\n", gid, result[gid],
+//         global_sum);
+  barrier(CLK_GLOBAL_MEM_FENCE);
+  for (i=0; i < 32; ++i) {
+    float value = input[gid+i];
+    global_sum += value;
+  }
+  result[gid] = result[gid] + global_sum;
+//  printf("after barrier GID %d result[gid] == %f global_sum == %f\n", gid, result[gid],
+//         global_sum);
+}
+)CL";
 
 int
 main(void)
@@ -93,7 +96,7 @@ main(void)
         // Pick first platform
         cl_context_properties cprops[] = {
             CL_CONTEXT_PLATFORM, (cl_context_properties)(platformList[0])(), 0};
-        cl::Context context(CL_DEVICE_TYPE_CPU | CL_DEVICE_TYPE_GPU, cprops);
+        cl::Context context(CL_DEVICE_TYPE_ALL, cprops);
 
         // Query the set of devices attched to the context
         std::vector<cl::Device> devices = context.getInfo<CL_CONTEXT_DEVICES>();
@@ -112,16 +115,16 @@ main(void)
 
         // Create buffer for A and copy host contents
         cl::Buffer aBuffer = cl::Buffer(
-            context, 
-            CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, 
-            BUFFER_SIZE * sizeof(float), 
+            context,
+            CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
+            BUFFER_SIZE * sizeof(float),
             (void *) &A[0]);
 
         // Create buffer for that uses the host ptr C
         cl::Buffer cBuffer = cl::Buffer(
-            context, 
-            CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR, 
-            WORK_ITEMS * sizeof(int), 
+            context,
+            CL_MEM_WRITE_ONLY | CL_MEM_USE_HOST_PTR,
+            WORK_ITEMS * sizeof(int),
             (void *) &R[0]);
 
         // Create kernel object
@@ -133,20 +136,19 @@ main(void)
 
         // Create command queue
         cl::CommandQueue queue(context, devices[0], 0);
- 
+
         // Do the work
         queue.enqueueNDRangeKernel(
-            kernel, 
-            cl::NullRange, 
+            kernel,
+            cl::NullRange,
             cl::NDRange(WORK_ITEMS),
             cl::NullRange);
- 
 
-        // Map cBuffer to host pointer. This enforces a sync with 
+        // Map cBuffer to host pointer. This enforces a sync with
         // the host backing space, remember we choose GPU device.
         int * output = (int *) queue.enqueueMapBuffer(
             cBuffer,
-            CL_TRUE, // block 
+            CL_TRUE, // block
             CL_MAP_READ,
             0,
             WORK_ITEMS * sizeof(int));
@@ -171,8 +173,8 @@ main(void)
             result = result + global_sum;
 
             if ((int)result != poclu_bswap_cl_int (dev_id, R[i])) {
-                std::cout 
-                    << "F(" << i << ": " << (int)result << " != " << R[i] 
+                std::cout
+                    << "F(" << i << ": " << (int)result << " != " << R[i]
                     << ") ";
                 ok = false;
             }
